unit DBTransparentImage;

{***************************************************************}
{                                                               }
{ Transparent DBImage control for Delphi                        }
{ Paints the bitmap transparently onto its background. Note     }
{ that its background is not itself seethrough, but has the     }
{ colour specified in its Color property, or the picture set in }
{ the BackgroundPic property.                                   }
{                                                               }
{ Use the SizePicture or SizePictureToFit in your datasource's  }
{ OnInsert method to enable drawing - you can't draw to a 0x0   }
{ bitmap!                                                       }
{                                                               }
{ Freeware by Deborah Pate                                      }
{ Version 1.22, October 1999                                    }
{ Limitations - no palette|QuickDraw stuff, sorry               } 
{                                                               }
{ Please send improvements, bug reports etc to me at:           }
{ Debs@djpate.freeserve.co.uk                                   }
{                                                               }
{***************************************************************}


interface

uses Windows, SysUtils, Classes, Controls, Messages, Graphics, Db, DBCtrls;

type

  TDBTransparentImage = class(TDBImage)
  private
    fBackgroundPic: TBitmap;
    fFocusRect: boolean;
    fFocusColor: TColor;
    fOnMouseExit: TNotifyEvent;
    fOnMouseEnter: TNotifyEvent;
    fTransparent: boolean;
    procedure SetOnMouseEnter(const Value: TNotifyEvent);
    procedure SetOnMouseExit(const Value: TNotifyEvent);
    procedure SetFocusColor(const Value: TColor);
    procedure SetFocusRect(const Value: boolean);
    procedure SetBackgroundPic(const Value: TBitmap);
    function GetCanvas: TCanvas;
   protected
    procedure CMMouseEnter (var Msg: TMessage); message CM_MOUSEENTER;
    procedure CMMouseExit (var Msg: TMessage); message CM_MOUSELEAVE;
    procedure DoMouseEnter; virtual;
    procedure DoMouseExit; virtual;
    procedure Paint; override;
    procedure SetTransparent(const Value: boolean);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SizePicture(const NewWidth, NewHeight: integer);
    procedure SizePictureToFit;                     // useful when you insert a record
    property PictureCanvas: TCanvas read GetCanvas; // just to make life a little easier
  published
    property BackgroundPic: TBitmap read FBackgroundPic write SetBackgroundPic;
    property FocusRect: boolean read FFocusRect write SetFocusRect default True;
    property FocusColor: TColor read FFocusColor write SetFocusColor default clWindowFrame;
    property OnMouseEnter: TNotifyEvent read fOnMouseEnter write SetOnMouseEnter;
    property OnMouseExit: TNotifyEvent read fOnMouseExit write SetOnMouseExit;
    property Transparent: boolean read fTransparent
                                  write SetTransparent default True;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Data Controls', [TDBTransparentImage]);
end;

{ TDBTransparentImage }

procedure TDBTransparentImage.CMMouseEnter(var Msg: TMessage);
begin
  inherited;
  DoMouseEnter;
end;

procedure TDBTransparentImage.CMMouseExit(var Msg: TMessage);
begin
  inherited;
  DoMouseExit;
end;

constructor TDBTransparentImage.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fBackgroundPic := TBitmap.Create;
  fFocusRect := True;
  fFocusColor := clWindowFrame;
  fTransparent := True;
  Color := clBtnFace;
end;

destructor TDBTransparentImage.Destroy;
begin
  FBackgroundPic.Free;
  inherited Destroy;
end;

procedure TDBTransparentImage.DoMouseEnter;
begin
  if Assigned(fOnMouseEnter) then
    fOnMouseEnter(Self);
end;

procedure TDBTransparentImage.DoMouseExit;
begin
  if Assigned(fOnMouseExit) then
    fOnMouseExit(Self);
end;

function TDBTransparentImage.GetCanvas: TCanvas;
begin
  Result := Picture.Bitmap.Canvas;
end;

procedure TDBTransparentImage.Paint;
var
  R: TRect;
  S: string;
  DrawPict: TPicture;
  TempBmp: TBitmap;
begin
  if ((Picture.Graphic <> nil) or (csPaintCopy in ControlState)) then begin
    DrawPict := TPicture.Create;
    try
      if (csPaintCopy in ControlState) and (Field <> nil)
        {$IFDEF VER120}
          and Field.IsBlob
        {$ENDIF} then
        DrawPict.Assign(Field)
      else DrawPict.Assign(Picture);

      TempBmp := TBitmap.Create;
      try
        TempBmp.Height := ClientHeight;
        TempBmp.Width := ClientWidth;
        with TempBmp.Canvas do begin
          if fBackgroundPic.Empty then
          begin
            Brush.Style := bsSolid;
            Brush.Color := Color;
            FillRect(ClientRect);
          end
          else StretchDraw(ClientRect, fBackgroundPic);

          if not ((DrawPict.Graphic = nil) or DrawPict.Graphic.Empty) then begin
            if Stretch then
              SetRect(R, 0, 0, ClientWidth, ClientHeight)
            else begin
              SetRect(R, 0, 0, DrawPict.Width, DrawPict.Height);
              if Center then OffsetRect(R, (ClientWidth - DrawPict.Width) div 2,
                (ClientHeight - DrawPict.Height) div 2);
            end;

            if fTransparent and (DrawPict.Graphic is TBitmap) then
            begin
              CopyMode := cmSrcAnd;
              StretchDraw(R, DrawPict.Bitmap);
              CopyMode := cmSrcCopy;
            end
            else StretchDraw(R, DrawPict.Graphic);
          end;
        end; //with
        Canvas.Draw(0, 0, TempBmp);
      finally
        TempBmp.Free;
      end;
    finally
      DrawPict.Free;
    end;
  end
  else begin
    Canvas.Font := Font;
    Canvas.Brush.Color := Color;
    if (Field <> nil) then
      S := Field.DisplayLabel
    else S := Name;
    S := '(' + S + ')';
    R := ClientRect;
    Canvas.TextRect(R, (R.Right - Canvas.TextWidth(S)) div 2, (R.Bottom - Canvas.TextHeight(S)) div 2, S);
  end;

  if (Focused and fFocusRect) then begin
    Canvas.Brush.Color := fFocusColor;
    Canvas.FrameRect(ClientRect);
  end;
end;

procedure TDBTransparentImage.SetBackgroundPic(const Value: TBitmap);
begin
  fBackgroundPic.Assign(Value);
  Invalidate;
end;

procedure TDBTransparentImage.SetFocusColor(const Value: TColor);
begin
  fFocusColor := Value;
  Invalidate;
end;

procedure TDBTransparentImage.SetFocusRect(const Value: boolean);
begin
  fFocusRect := Value;
  Invalidate;
end;

procedure TDBTransparentImage.SetOnMouseEnter(const Value: TNotifyEvent);
begin
  fOnMouseEnter := Value;
end;

procedure TDBTransparentImage.SetOnMouseExit(const Value: TNotifyEvent);
begin
  fOnMouseExit := Value;
end;

procedure TDBTransparentImage.SetTransparent(const Value: boolean);
begin
  fTransparent := Value;
  Invalidate;
end;

procedure TDBTransparentImage.SizePicture(const NewWidth, NewHeight: integer);
var
  OldHeight, OldWidth: integer;
  OldColour: TColor;
  OldBrushStyle: TBRushStyle;
  NewRect: TRect;
begin
  OldHeight := Picture.Bitmap.Height;
  OldWidth := Picture.Bitmap.Width;
  Picture.Bitmap.Height := NewHeight;
  Picture.Bitmap.Width := NewWidth;
  if (OldHeight < NewHeight) or (OldWidth < Width) then
  begin
    OldColour := PictureCanvas.Brush.Color;
    OldBrushStyle := PictureCanvas.Brush.Style;
    PictureCanvas.Brush.Style := bsSolid;
    PictureCanvas.Brush.Color := PictureCanvas.Pixels[0,0];
    if OldHeight < NewHeight then
    begin
      NewRect := Rect(0, OldHeight, NewWidth, NewHeight);
      PictureCanvas.FillRect(NewRect);
    end;
    if OldWidth < NewWidth then
    begin
      NewRect := Rect(OldWidth, 0, NewWidth, NewHeight);
      PictureCanvas.FillRect(NewRect);
    end;
    PictureCanvas.Brush.Style := OldBrushStyle;
    PictureCanvas.Brush.Color := OldColour;
  end;
end;

procedure TDBTransparentImage.SizePictureToFit;
begin
  SizePicture(Width, Height);
end;

end.
